require 'flappy/version'
require 'thor'

module Flappy
  class CLI < Thor
    class_option :firToken,   type: :string,  aliases: '-T', desc: "User's API Token at fir.im"
    class_option :logfile, type: :string,  aliases: '-L', desc: 'Path to writable logfile'
    class_option :verbose, type: :boolean, aliases: '-V', desc: 'Show verbose', default: true
    class_option :quiet,   type: :boolean, aliases: '-q', desc: '安静模式，只显示必要的信息'
    class_option :help,    type: :boolean, aliases: '-h', desc: 'Show this help message and quit'

    desc 'build_apk BUILD_DIR', 'Build Android app (alias: `ba`).'
    long_desc <<-LONGDESC
      `build_apk` command will auto build your project to an apk package
      and it also can auto publish your built apk to fir.im if use `-p` option.
      Internally, it use `gradle` to accomplish these things, use `gradle --help` to get more information.

      Example:

      $ flappy ba <project dir> [-o <apk output dir> -c <changelog> -p -Q -T <your api token>]

      $ flappy ba <project dir> [-f <flavor> -o <apk output dir> -c <changelog> -p -Q -T <your api token>]

      $ flappy ba <git ssh url> [-B develop -o <apk output dir> -c <changelog> -p -Q -T <your api token>]
    LONGDESC
    map ['ba'] => :build_apk
    method_option :branch,    type: :string,  aliases: '-B', desc: 'Set branch if project is a git repo, the default is `master`'
    method_option :output,    type: :string,  aliases: '-o', desc: 'APK output path, the default is: BUILD_DIR/build/outputs/apk'
    method_option :publish,   type: :boolean, aliases: '-p', desc: 'true/false if publish to fir.im'
    method_option :productFlavor,    type: :string,  aliases: '-f', desc: 'Set flavor if have productFlavors'
    method_option :buildType,     type: :boolean, aliases: '-d', desc: 'Set buildType if debug or release', default: false
    method_option :short,     type: :string,  aliases: '-s', desc: 'Set custom short link if publish to fir.im'
    method_option :name,      type: :string,  aliases: '-n', desc: 'Set custom apk name when builded'
    method_option :changelog, type: :string,  aliases: '-c', desc: 'Set changelog if publish to fir.im, support string/file'
    method_option :qrcode,    type: :boolean, aliases: '-Q', desc: 'Generate qrcode'
    method_option :open,      type: :boolean, desc: 'true/false if open for everyone, the default is: true', default: true
    method_option :password,  type: :string,  desc: 'Set password for app'
    def build_apk(*args)
      prepare :build_apk

      Flappy.build_apk(*args, options)
    end

    desc 'build_ipa BUILD_DIR [options] [settings]', 'Build iOS app (alias: `bi`).'
    long_desc <<-LONGDESC
      `build_ipa` command will auto build your project/workspace to an ipa package

      Example:

      $ flappy bi <workspace dir> -S <scheme name> [-C <configuration>] [-t <target name>] [-o <ipa output dir>] [settings] [-c <changelog>] [-p -Q -T <your api token>]
      $ flappy bi <project dir>                    [-C <configuration>] [-t <target name>] [-o <ipa output dir>] [settings] [-c <changelog>] [-p -Q -T <your api token>]
      $ flappy bi <git ssh url> [-B <branch>]      [-C <configuration>] [-t <target name>] [-o <ipa output dir>] 

    LONGDESC

    map ['bi'] => :build_ipa
    ######### pod
    method_option :'update-pod',              type: :boolean,   desc: '是否更新pod`'
    method_option :'no-repo-update',          type: :boolean,   desc: '安装pod时不更新cocoapod库'
    method_option :'only-update-souche-pod',  type: :boolean,   desc: '是否只更新车牛pod'
    method_option :'pod-env-test',            type: :boolean,   desc: '是否依赖测试环境pod'

    ######### other
    method_option :open,          type: :boolean,    desc: '打包完成后是否打开文件夹'
    method_option :'no-savelog',          type: :boolean,    desc: '是否保存打包日志'

    ######## 环境
    method_option :'https',       type: :string,     desc: "是否开启htts, 'Y' or 'N'"
    method_option :'env',         type: :string,     desc: '主环境'
    method_option :'port',        type: :string,     desc: '主环境端口'
    method_option :'pay-env',     type: :string,     desc: '支付环境'

    ######## package
    method_option :branch,        type: :string,   aliases: '-B',   desc: '打包分支，默认为master'
    method_option :scheme,        type: :string,   aliases: '-S',   desc: 'build the scheme NAME'
    method_option :configuration, type: :string,   aliases: '-C',   desc: 'use the build configuration NAME for building each target'
    method_option :target,        type: :string,   aliases: '-t',   desc: 'build the target NAME'
    method_option :output,        type: :string,   aliases: '-o',   desc: 'IPA等打包信息的存放路径'

    def build_ipa(*args)
      prepare :build_ipa
      Flappy.build_ipa(*args, options)
    end

    desc 'archive_ipa', '归档build文件为ipa'
    def archive_ipa(*args)
      prepare :archive_ipa
      Flappy.archive_ipa(*args, options)
    end


    desc 'publish APP_FILE_PATH', 'Publish iOS/Android app to fir.im, support ipa/apk file (aliases: `p`).'
    long_desc <<-LONGDESC
      `publish` command will publish your app file to fir.im, also the command support to publish app's short & changelog.

      Example:

      $ flappy p <app file path> [-c <changelog> -s <custom short link> -Q -T <your api token>]

      $ flappy p <app file path> [-c <changelog> -s <custom short link> --password=123456 --open=false -Q -T <your api token>]

      $ flappy p <app file path> [-c <changelog> -s <custom short link> -m <mapping file path> -P <bughd project id> -Q -T <your api token>]
    LONGDESC
    map 'p' => :publish
    method_option :short,       type: :string,  aliases: '-s', desc: 'Set custom short link'
    method_option :changelog,   type: :string,  aliases: '-c', desc: 'Set changelog'
    method_option :qrcode,      type: :boolean, aliases: '-Q', desc: 'Generate qrcode'
    method_option :mappingfile, type: :string,  aliases: '-m', desc: 'App mapping file'
    method_option :proj,        type: :string,  aliases: '-P', desc: 'Project id in BugHD.com if upload app mapping file'
    method_option :open,        type: :boolean, desc: 'true/false if open for everyone'
    method_option :password,    type: :string,  desc: 'Set password for app'
    def publish(*args)
      prepare :publish

      Flappy.publish(*args, options)
    end

    desc 'help', 'Describe available commands or one specific command (aliases: `h`).'
    map Thor::HELP_MAPPINGS => :help
    def help(command = nil, subcommand = false)
      super
    end

    no_commands do
      def invoke_command(command, *args)
        logfile = options[:logfile].blank? ? STDOUT : options[:logfile]
        logfile = '/dev/null' if options[:quiet]

        Flappy.logger       = Logger.new(logfile)
        Flappy.logger.level = options[:verbose] ? Logger::INFO : Logger::ERROR
        super
      end
    end

    private

    def prepare(task)
      if options.help?
        help(task.to_s)
        fail SystemExit
      end
      $DEBUG = true if ENV['DEBUG']
    end
  end
end
