# encoding: utf-8

module Flappy
  module IpaInfo
    class Ipa
      # include Parser::Common

      def initialize(path)
        @path = path   # ipa包的路径
      end

      def contents   # contents为ipa解压路径
        return @contents if @contents

        @contents = "#{Dir.tmpdir}/#{Time.now.strftime('%Y%m%d%H%M%S')}_ipa_files"
        Zip::File.open(@path) do |zip_file|  # zip_file为待解压的文件路径
          # puts "zipfile: #{zip_file}"
          zip_file.each do |f|    # f为解压出来的文件
            # puts "file: #{f}"
            f_path = File.join(@contents, f.name)   # f_path为解压出来的文件路径
            # puts "path: #{f_path}"
            # FileUtils.mkdir_p(File.dirname(f_path))  # File.dirname(f_path)为所有的文件夹、资源束文件；这里建立压缩包里面的每个文件夹，这一步可以省略的
            # puts "dir: #{File.dirname(f_path)}"
            zip_file.extract(f, f_path) unless File.exist?(f_path)  # 解压文件到相应位置
          end
        end
        @contents
      end

      def app_path
        @app_path ||= Dir.glob(File.join(contents, 'Payload', '*.app')).first
      end

      def app
        @app ||= App.new(app_path, is_stored)
      end

      def cleanup
        return unless @contents
        FileUtils.rm_rf(@contents)
        @contents = nil
      end

      def metadata
        return unless has_metadata?
        begin
          @metadata ||= CFPropertyList.native_types(CFPropertyList::List.new(file: metadata_path).value)
        rescue CFFormatError
          @metadata = {}
        end
      end

      def has_metadata?
        File.file?(metadata_path)
      end

      def metadata_path
        @metadata_path ||= File.join(contents, 'iTunesMetadata.plist')
      end

      def is_stored   # 是否是appstore包
        has_metadata? ? true : false
      end
    end


    class App
      def initialize(path, is_stored = false)
        @path      = path       # app的路径
        @is_stored = is_stored
      end

      def full_info(options)
        # if options.fetch(:full_info, false)
        #   basic_info.merge!(icons: tmp_icons)
        # end

        basic_info
      end

      def basic_info
        @basic_info ||= {
            type:              'ios',
            identifier:        identifier,
            name:              name,
            display_name:      display_name,
            build:             version.to_s,
            version:           short_version.to_s,
            devices:           devices,
            release_type:      release_type,
            distribution_name: distribution_name
        }
      end

      def info
        @info ||= CFPropertyList.native_types(
            CFPropertyList::List.new(file: File.join(@path, 'Info.plist')).value)
      end

      def identifier
        info['CFBundleIdentifier']
      end

      def name
        info['CFBundleName']
      end

      def display_name
        info['CFBundleDisplayName']
      end

      def version
        info['CFBundleVersion']
      end

      def short_version
        info['CFBundleShortVersionString']
      end

      def mobileprovision_path
        @mobileprovision_path ||= File.join(@path, 'embedded.mobileprovision')
      end

      def has_mobileprovision?
        File.file? mobileprovision_path
      end

      def mobileprovision
        return unless has_mobileprovision?
        return @mobileprovision if @mobileprovision

        cmd = "security cms -D -i \"#{mobileprovision_path}\""
        begin
          @mobileprovision = CFPropertyList.native_types(CFPropertyList::List.new(data: `#{cmd}`).value)
        rescue CFFormatError
          @mobileprovision = {}
        end
      end

      def devices
        mobileprovision['ProvisionedDevices'] if has_mobileprovision?
      end

      def hide_developer_certificates
        mobileprovision.delete('DeveloperCertificates') if has_mobileprovision?
      end

      def distribution_name
        "#{mobileprovision['Name']} - #{mobileprovision['TeamName']}" if has_mobileprovision?
      end

      def release_type
        if @is_stored
          'Release'
        else
          if has_mobileprovision?
            if devices
              'AdHoc'
            else
              'InHouse'
            end
          end
        end
      end

      def icons
        @icons ||= begin
          icons = []
          info['CFBundleIcons']['CFBundlePrimaryIcon']['CFBundleIconFiles'].each do |name|
            icons << get_image_path(name)
            icons << get_image_path("#{name}@2x")
            icons << get_image_path("#{name}@3x")
          end
          icons.delete_if &:!
        rescue NoMethodError
          []
        end

        begin
          info['CFBundleIcons~ipad']['CFBundlePrimaryIcon']['CFBundleIconFiles'].each do |name|
            icons << get_image_path("#{name}~ipad")
            icons << get_image_path("#{name}@2x~ipad")
          end
            icons.delete_if &:!
        rescue NoMethodError
          []
        end
      end

      def get_image_path(name)
        path = File.join(@path, "#{name}.png")
        return nil unless File.exist?(path)
        path
      end
    end

  end
end
